/* Copyright (c) 2008 terrestris GmbH & Co KG, published under the Clear BSD
 * license.  See http://svn.openlayers.org/trunk/openlayers/license.txt for the
 * full text of the license. */
/**
 * Namespace: OpenLayers.Layer.SOS
/**
 * @requires OpenLayers/Layer/Vector.js
 * @requires OpenLayers/Layer/Markers.js
 */

/**
 * Class: OpenLayers.Layer.SOS
 *
 * Inherits from:
 *  - <OpenLayers.Layer.Vector>
 *  - <OpenLayers.Layer.Markers>
 *
 * date: 10. August 2009
 * author: Dominik Helle, Till Adams
 * version: 0.6
 *
 *
/**

OpenLayers.Layer.SOS = OpenLayers.Class( OpenLayers.Layer.Markers,

    /**
     * APIProperty: isBaseLayer
     * {Boolean} SOS layer is not a base layer by default.
     */
    isBaseLayer: false,

    /**
     * Property: tile
     * {<OpenLayers.Tile.SOS>}
     */
    tile: null,

      /**
       * Property: url
       * {String} store url of text file
       */
     url : null,

     /**
       * Property: url
       * {String} store url of text file
       */

	  format: null,
     /**
       * Property: features
       * {Array(<OpenLayers.Feature>)}
       */
     features: null,

     /**
   	  * APIProperty: popupSize
  	  * {<OpenLayers.Size>} This determines the size of Sensor popups. If
   	  * not provided, defaults to 250px by 120px.
   	  */
	   popupSize: null,

    /**
     * Constructor: OpenLayers.Layer.SOS
     *
     * Parameters:
     * name - {String}
     * url - {String}
     * params - {Object}
     * options - {Object}
     */
    initialize: function(name, url, params, options) {

      // initialisieren des Markers
	    OpenLayers.Layer.Markers.prototype.initialize.apply(this, [name, options]);

      this.title = name;
      this.url = url;
	    this.params = params;

	  // Laden der klassen die f??r das erstellen und parsen des XML zust??ndig ist
  	  this.format = new OpenLayers.Format.SOS();

      this.features = [];
      this.events.triggerEvent("loadstart");
      this.loadSensor();
    },

    /**
     * Method: destroy
     */
    destroy: function() {
        OpenLayers.Layer.Markers.prototype.destroy.apply(this, arguments);
        this.clearFeatures();
        this.features = null;
    },


    /**
     * Methode: loadSensor
     *
     *
     */

    loadSensor: function(xmlRequest) {
      if (!this.loaded) {
         this.events.triggerEvent("loadstart");

		// erstellen des GetObservation-Requests
         var xmlRequest = this.format.write_GetObservation(this.params);

		// GetObservation Request an den SOS senden
		   var request = OpenLayers.Request.POST({
		    url: this.url,
		    data: xmlRequest,
			  callback: this.parseData,
			  scope: this
		   });
	     }
     },


    /**
     * Method: parseData
     * Parse the data returned from the Events call.
     *
     * Parameters:
     * ajaxRequest - {XMLHttpRequest}
     */
    parseData: function(ajaxRequest) {
      	var description = "";
	   	var data = {};



  		var doc = ajaxRequest.responseXML;
        if (!doc || !doc.documentElement) {
             doc = OpenLayers.Format.XML.prototype.read(ajaxRequest.responseText);
        }


		// create Pop Up window
	    data.popupSize = this.popupSize ?
                         this.popupSize.clone() :
                         new OpenLayers.Size(230, 140);

	// if user defined Icon use this instead of standards icon
       	data.icon = this.icon == null ?
              OpenLayers.Marker.defaultIcon() :
	this.icon.clone();
	data.title =  this.title;

	// read informations from O&M
    	var location = this.format.read_LatLon(doc);
 	var time = this.format.read_TimeStamp(doc);
	var observerproperty = this.format.read_ObserverdProperty(doc);
	var value = this.format.read_Value(doc);

	for (var i=0; i < observerproperty.length; i++) {
		 description += "property: <strong>" + observerproperty[i] +"</strong><br/>" +
		    "time:    <strong>" +     time[i]         +"</strong><br/>" +
	 	    "value:   <strong>" +     value[i]        +"</strong><br/>";
		};

		// put PopUp together
		contentHTML = 	"<div id='popUp' class='sospaginated'>"+
				"<a class='closeButton' href='#'>Close</a>"+
				"</div>";
		contentHTML += "<h2 class='labelHeader'>"+ this.title +"</h2>";


	contentHTML += '<div style="" class="olLayerSOSDescription">';
	contentHTML +=  description;
	contentHTML +=  '</div>';

	data['popupContentHTML'] = contentHTML;

        // add new Feature
    	var feature = new OpenLayers.Feature(this, location, data);
        this.features.push(feature);

	// create new Marker & add event markerClick
	var marker = feature.createMarker();
        marker.events.register('click', feature, this.markerClick);
        this.addMarker(marker);

	// trigger event ende
        this.events.triggerEvent("loadend");
	},


    /**
     * Method: markerClick
     *
     * Parameters:
     * evt - {Event}
     */
    markerClick: function(evt) {
		// sameMarkerClicked
        var sameMarkerClicked = (this == this.layer.selectedFeature);
        this.layer.selectedFeature = (!sameMarkerClicked) ? this : null;

	// delete if there is any popup still open
        for(var i=0; i < this.layer.map.popups.length; i++) {
            this.layer.map.removePopup(this.layer.map.popups[i]);
        }

	// create new Marker
        if (!sameMarkerClicked) {
            var popup = this.createPopup();
            OpenLayers.Event.observe(popup.div, "click",
                OpenLayers.Function.bind(function() {
                    for(var i=0; i < this.layer.map.popups.length; i++) {
                        this.layer.map.removePopup(this.layer.map.popups[i]);
                    }
                }, this)
            );
            this.layer.map.addPopup(popup);
        }
        OpenLayers.Event.stop(evt);
    },

    /**
     * Method: clearFeatures
     * Destroy all features in this layer.
     */
    clearFeatures: function() {
        if (this.features != null) {
            while(this.features.length > 0) {
                var feature = this.features[0];
                OpenLayers.Util.removeItem(this.features, feature);
                feature.destroy();
            }
        }
    },

    CLASS_NAME: "OpenLayers.Layer.SOS"
// hierhin geh?rt noch eine geschwungene Klammer aber er kompiliert so nicht
);
